from django.http import HttpResponse
from django.urls import include, path


def test_view(request):
    return HttpResponse("not a real view")


urlpatterns = [
    path("", test_view),
    path(
        "accounts/",
        include(("okta_oauth2.urls", "okta_oauth2"), namespace="okta_oauth2"),
    ),
]



class InvalidToken(Exception):
    """ Base exception for an invalid token """

    pass


class InvalidTokenSignature(InvalidToken):
    """ Token signatures doesn't validate """

    pass


class IssuerDoesNotMatch(InvalidToken):
    """ Token Issuer doesn't match expected issuer """

    pass


class InvalidClientID(InvalidToken):
    """ Token ClientID doesn't match expected Client ID """

    pass


class TokenExpired(InvalidToken):
    """ Token expiration time is in the past """

    pass


class TokenTooFarAway(InvalidToken):
    """ The received token is not valid until too far in the future. """

    pass


class NonceDoesNotMatch(InvalidToken):
    """ Token nonce does not match expected nonce """

    pass


class TokenRequestFailed(Exception):
    """ The request to the token api endpoint has failed. """

    pass



from django.utils.timezone import now
from jose import jwt
from okta_oauth2.conf import Config


def build_token(
    aud=None,
    auth_time=None,
    exp=None,
    iat=None,
    iss=None,
    sub=None,
    nonce="defaultnonce",
):
    config = Config()

    current_timestamp = now().timestamp()
    iat_offset = 2

    claims = {
        "amr": ["pwd"],
        "at_hash": "notarealhash",
        "aud": aud if aud else config.client_id,
        "auth_time": auth_time if auth_time else current_timestamp,
        "email": "fakemail@notreal.com",
        "exp": exp if exp else current_timestamp + iat_offset + 3600,
        "iat": iat if iat else current_timestamp + iat_offset,
        "idp": aud if aud else config.client_id,
        "iss": iss if iss else config.issuer,
        "jti": "randomid",
        "name": "A User",
        "nonce": nonce,
        "preferred_username": "auser",
        "sub": sub if sub else config.client_id,
        "ver": 1,
    }

    headers = {"kid": "1A234567890"}

    return jwt.encode(claims, "secret", headers=headers, algorithm="HS256")



import logging

from django.contrib.auth import authenticate, login as auth_login, logout as auth_logout
from django.http import HttpResponseBadRequest, HttpResponseRedirect
from django.shortcuts import redirect, render
from django.urls import reverse
from django.urls.exceptions import NoReverseMatch

from .conf import Config

logger = logging.getLogger(__name__)


def login(request):
    config = Config()

    okta_config = {
        "clientId": config.client_id,
        "url": config.org_url,
        "redirectUri": str(config.redirect_uri),
        "scope": config.scopes,
        "issuer": config.issuer,
    }
    response = render(request, "okta_oauth2/login.html", {"config": okta_config})

    _delete_cookies(response)

    return response


def callback(request):
    config = Config()

    if request.POST:
        return HttpResponseBadRequest("Method not supported")

    code = request.GET["code"]
    state = request.GET["state"]

    # Get state and nonce from cookie
    cookie_state = request.COOKIES["okta-oauth-state"]
    cookie_nonce = request.COOKIES["okta-oauth-nonce"]

    # Verify state
    if state != cookie_state:
        return HttpResponseBadRequest(
            "Value {} does not match the assigned state".format(state)
        )

    user = authenticate(request, auth_code=code, nonce=cookie_nonce)

    if user is None:
        return redirect(reverse("okta_oauth2:login"))

    auth_login(request, user)

    try:
        redirect_url = reverse(config.login_redirect_url)
    except NoReverseMatch:
        redirect_url = config.login_redirect_url

    return redirect(redirect_url)


def logout(request):
    auth_logout(request)
    return HttpResponseRedirect(reverse("okta_oauth2:login"))


def _delete_cookies(response):
    # The Okta Signin Widget/Javascript SDK aka "Auth-JS" automatically generates
    # state and nonce and stores them in cookies. Delete authJS/widget cookies
    response.set_cookie("okta-oauth-state", "", max_age=1)
    response.set_cookie("okta-oauth-nonce", "", max_age=1)
    response.set_cookie("okta-oauth-redirect-params", "", max_age=1)



SECRET_KEY = "imasecretlol"

DATABASES = {"default": {"NAME": "test.db", "ENGINE": "django.db.backends.sqlite3"}}

INSTALLED_APPS = (
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "okta_oauth2.apps.OktaOauth2Config",
)

OKTA_AUTH = {
    "ORG_URL": "https://test.okta.notreal/",
    "ISSUER": "https://test.okta.notreal/oauth2/default",
    "CLIENT_ID": "not-a-real-id",
    "CLIENT_SECRET": "not-a-real-secret",
    "REDIRECT_URI": "http://localhost:8000/accounts/callback/",
}

ROOT_URLCONF = "okta_oauth2.tests.urls"
